@@ -18,6 +18,7 @@ test_that("HclustParam constructor and utilities work correctly", {
     expect_output(show(HclustParam(method="average")), "average")
     expect_output(show(HclustParam(cut.dynamic=TRUE)), "cutreeDynamic")
     expect_output(show(HclustParam(cut.fun=identity)), "custom")
+    expect_output(show(HclustParam()), "hclust")
 })
 
 test_that("HclustParam validity works correctly", {
@@ -61,3 +62,16 @@ test_that("clusterRows works with the dynamic tree cut", {
     expect_identical(length(out), nrow(m))
 })
 
+# Vegan-dependant test
+test_that("dist.fun parameter works correctly", {
+    m <- matrix(runif(1000), ncol=10)
+    dist_result <- clusterRows(m, HclustParam(metric = "euclidean"))
+    vegdist_result <- clusterRows(m, HclustParam(metric = "euclidean", dist.fun = vegan::vegdist))
+    expect_identical(dist_result, vegdist_result)
+    
+    vegdist_result2 <- clusterRows(m, HclustParam(metric = "canberra", dist.fun = vegan::vegdist), full = TRUE)
+    res_dist_matrix <- vegdist_result2$objects$dist
+    original_dist_matrix <- vegan::vegdist(m, "canberra")
+    expect_setequal(res_dist_matrix, original_dist_matrix)
+})
+
